I have tested wp2latex (C version) using the following OS/compiler
combinations:
1. 386 Xenix 2.3.2/cc & gcc
2. DOS/Xenix cc -dos & MSC 5.1
3. SunOS 4.0.3 & 4.1(SPARC)/cc

The DOS versions require a large model compilation and a stack size of
0x3000 in order to run.

The SUN version runs exceedingly slowly. I don't know why. (By slow I mean
a 4.77MHz XT is *faster*) I have profiled the code and seems to be spending
a lot of time in lseek. Any ideas would be welcome.

Define HAVE_P2C in the Makefile and correct the libraries required if you 
have p2c 1.14 or higher installed.
			Share and enjoy,
				Glenn

glenn@qed.physics.su.oz.au
